// Copyright (c) 2013 by István Váradi

// This file is part of VSCPL, a simple cross-platform utility library

// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:

// 1. Redistributions of source code must retain the above copyright notice, this
//    list of conditions and the following disclaimer.
// 2. Redistributions in binary form must reproduce the above copyright notice,
//    this list of conditions and the following disclaimer in the documentation
//    and/or other materials provided with the distribution.

// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
// ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

// The views and conclusions contained in the software and documentation are those
// of the authors and should not be interpreted as representing official policies,
// either expressed or implied, of the FreeBSD Project.

//------------------------------------------------------------------------------

#include "LocalServerSocket.h"

#include <cstdio>

#include <pwd.h>
#include <unistd.h>
#include <poll.h>

#include <sys/socket.h>

//------------------------------------------------------------------------------

using hu::varadiistvan::scpl::io::LocalServerSocket;

//------------------------------------------------------------------------------

void LocalServerSocket::setupPath(struct sockaddr_un& addr, const char* name)
{
#ifndef P_tmpdir
#define P_tmpdir "/tmp"
#endif
    struct passwd* passwd = getpwuid(geteuid());
    const char* userName = (passwd==0) ? "unknown" : passwd->pw_name;
    snprintf(addr.sun_path, sizeof(addr.sun_path), "%s/%s-%s",
             P_tmpdir, name, userName);
}

//------------------------------------------------------------------------------

LocalServerSocket::LocalServerSocket(const char* name, Waiter* waiter) :
    ServerSocket(AF_UNIX, SOCK_STREAM, waiter),
    acceptor(this)
{
    sun.sun_family = AF_UNIX;
    setupPath(sun, name);

    unlink(sun.sun_path);

    if (bind(reinterpret_cast<struct sockaddr*>(&sun), sizeof(sun), false)<0) {
        setErrorCode(errno);
        return;
    }

    if (listen()<0) {
        setErrorCode(errno);
        return;
    }
}

//------------------------------------------------------------------------------

LocalServerSocket::~LocalServerSocket()
{
    unlink(sun.sun_path);
}

//------------------------------------------------------------------------------

bool LocalServerSocket::isReady()
{
    return false;
}

//------------------------------------------------------------------------------

void LocalServerSocket::handleEvents(short events)
{
    if ((events&POLLIN)==POLLIN) {
        acceptor.accept();
    }
}

//------------------------------------------------------------------------------

// Local Variables:
// mode: C++
// c-basic-offset: 4
// indent-tabs-mode: nil
// End:
